# **************************************************
#
#  The kraken entry data
#
# **************************************************

import pickle
import time
import sys
import os.path
import string
import urllib
import strptime
import random
import traceback
import logging

from rdflib.Literal import Literal
from rdflib.BNode import BNode
from rdflib.Identifier import Identifier
from rdflib.Namespace import Namespace
from rdflib.constants import TYPE, ABOUT
from rdflib.TripleStore import TripleStore
from rdflib.URIRef import URIRef

import kraken.tools.TextUi as TextUi
import kraken.Constants
import kraken.entrytypes.Utilities
from EntryBase import EntryBase
from kraken.tools.UniqueIdGenerator import getUniqueId
from kraken.entrytypes.bibtex.BibTeXEntry import BibTeXEntry
from kraken.entrytypes.opinion.Opinion import Opinion
from TypicalNamespaces import *

KRAKEN_ENTRY = Namespace("http://localhost/krakenEntrySchema/")

class Entry(EntryBase):            
    '''
    The entry class
    '''
    

    
# RDF mappings of the id's in the class
    mappings = {}
    mappings["label"]          = NS_KRAKEN["label"]
    mappings["directoryName"]  = NS_KRAKEN["directoryName"]
    mappings["title"]          = NS_DC["title"]
    mappings["description"]    = NS_DC["description"]
    mappings["date"]           = NS_DC["date"]
    mappings["localFileNames"] = NS_KRAKEN["localFileNames"]
    mappings["originalURIs"]   = NS_KRAKEN["originalURIs"]
    mappings["keywords"]       = NS_KRAKEN["keywords"]
    # mappings["size"]           = NS_KRAKEN["size"]
            
    def __init__(self):
       # the label. This is a fictive URL: kraken://
       # self.label = "kraken://" + getUniqueId()
       self.label = "#" + getUniqueId()
       # the name of the directory inside the kraken. This is considered
       # to be mildly significant
       self.directoryName = ""
       # the title of the enty
       self.title = ""
       # short textual description of the entry
       self.description = ""
       # the date when it was created
       self.date = time.localtime()
       # the name of top level files. If there is more than one, it should
       # as a continuation. There can be extra files (icons etc) which do not
       # appear here
       self.localFileNames = []
       # the original URLs of the entries
       self.originalURIs = []
       # the keywords
       self.keywords = [] # comma separated
       
       # Non-persistent data (is not serialized in RDF)       
       # the kraken dir name (for multiple directories)
       self.krakenDirName = ""
       self.opinion = None
       
    def listPropertyNames(self):
        return self.mappings.keys()
           
    def getResourceURI(self):
        return URIRef(self.label)

    def getMapping(self, field):
        return self.mappings[field]

    def getRdfClass(self):
        return URIRef(NS_KRAKONT["KrakenEntry"])                    

    def getProperty(self, propertyName):
        return self.__dict__[propertyName]

    def addProperty(self, propertyName, propertyValue):
        self.__dict__[propertyName] = propertyValue        
        
     # **************************************************
     #   fillin data from runtime
     #   this refers to the annotation data, not the original stuff, etc
     # **************************************************
    def fillinEntryFromRuntime(self, runtime, system, force = 0): 
        self.directoryName = runtime.entryDir
        if self.krakenDirName == "":
            self.krakenDirName = system.getProperty("destinationDir") 
        if force or runtime.title != "":
            self.title = runtime.title
        if force or runtime.description != "":     
            self.description = runtime.description
        if force or runtime.keywords != []:  
            self.keywords = runtime.keywords
        if force or runtime.date != 0:
            self.date = runtime.date

    #
    #  Command line based editing of an entry - mainly for annotation, etc
    #            
    def edit(self, metadata = None, isAnnotation = 1):
        print "Label = " + self.label
        self.title = TextUi.inputString("Title", self.title)
        self.description = TextUi.inputString("Description", self.description)
        if metadata != None:
           self.keywords = TextUi.inputSequence("Keywords", self.keywords, metadata.keywords)
        else:
           self.keywords = TextUi.inputSequence("Keywords", self.keywords)
        # the select files is only for the annotation step, because at first is not present   
        if isAnnotation:   
          self.localFileNames = TextUi.selectFilesFromDirectory(
              os.path.join(self.krakenDirName, self.directoryName), self.localFileNames)
        # the originalURI's
        self.originalURIs = TextUi.inputSequence("Remote URI's:", self.originalURIs)
        temp = TextUi.inputWithDefault("Date", time.strftime("%a, %d %b %Y %H:%M:%S +0000", self.date))
        self.date = strptime.strptime(temp, "%a, %d %b %Y %H:%M:%S +0000")
        kraken.entrytypes.Utilities.edit(self, metadata)
        
   
    #
    #  Checks if the metadata dir exists and creates it if it doesn't
    #
    def __prepareMetadataDir(self, dirName):
        metadataDir = os.path.join(dirName, kraken.Constants.DIRNAME_METADATA)
        if os.path.isdir(metadataDir):
            return
        os.mkdir(metadataDir)       
       
    def writeRDF(self, dirName):
       self.__prepareMetadataDir(dirName)
       self.writeEntryToRdf(getEntryRdfFile(dirName))
       if (self.opinion != None):
           opinionfile = getEntryOpinionRdfFile(dirName)
           self.opinion.writeEntryToRdf(opinionfile)

       
    def __str__(self): 
        x = "Entry:\n"
        x = x + "\tlabel = " + self.label
        x = x + "\n\ttitle = " + self.title
        x = x + "\n\tdescription = " + self.description
        x = x + "\n\tdate = " + time.strftime("%a, %d %b %Y %H:%M:%S +0000", self.date)
        x = x + "\n\tlocalFileNames = " + str(self.localFileNames)
        x = x + "\n\toriginalURIs = " + str(self.originalURIs)
        x = x + "\n\tkeywords = " + str(self.keywords)
        return x
        
def _getEntryAuxFileName(dirName, filename):
    return os.path.join(dirName, kraken.Constants.DIRNAME_METADATA, filename)
        
def getEntryRdfFile(dirName):
    return _getEntryAuxFileName(dirName, kraken.Constants.FILENAME_ENTRY_RDF)

def getEntryRdfFileOld(dirName):
    return _getEntryAuxFileName(dirName, kraken.Constants.FILENAME_ENTRY_RDF_OLD)
        
def getEntryBibTeXRdfFile(dirName):
    return _getEntryAuxFileName(dirName, "bibtex.rdf")

def getEntryBibTeXFile(dirName):
    return _getEntryAuxFileName(dirName, "bibtex.bib")

def getEntryOpinionRdfFile(dirName):
    return _getEntryAuxFileName(dirName, "opinion.rdf")
        
#
#  Loading an entry from the file. FIXME: this is not quity good because it 
#  works only for the destination dir.
#        
def loadKraEntry(system, dirName):
  try:    
    fileToOpen = getEntryRdfFile(dirName)
    temp = Entry()
    temp.readEntryFromRdf(fileToOpen)
    # temporary, for rewrite and fix the size stuff
    #temp.writeEntryToRdf(getEntryRdfFile(dirName))
    
    # FIXME: this should not come here, but well...
    # BIBTEX loading and converting
    bibtexfile = getEntryBibTeXFile(dirName)
    if os.path.isfile(bibtexfile):
        #print "This stuff has Bibtex"
        bibentry = BibTeXEntry()
        bibentry.resourceURI = temp.getResourceURI()
        bibentry.loadBibTeXFile(bibtexfile)
        #print bibentry
        bibentry.writeEntryToRdf(bibtexfile[0:-4] + ".rdf")    
        temp.bibtex = bibentry
    # END OF BIBTEX STUFF
    # OPINION LOADING
    opinionfile = getEntryOpinionRdfFile(dirName)
    if os.path.isfile(opinionfile):
        temp.opinion = Opinion()        
        temp.opinion.readEntryFromRdf(opinionfile)
        temp.opinion.resourceURI = temp.getResourceURI()
    # END OPINION        
    return temp
  except:
      traceback.print_exc()
      try:
        fileToOpen = getEntryRdfFileOld(dirName)
        temp.loadRDF(fileToOpen)
        logging.info("Found only in the old format, let's move it to the new one:" + dirName)
        temp.writeEntryToRdf(getEntryRdfFile(dirName))
        return temp
      except:
        logging.warning("Can not open file: "+dirName + "\n" + str(sys.exc_info()))
        return None
        
        

