import os.path
import urllib
from types import *
import time
import sys
import string
import logging

from rdflib.URIRef import URIRef
from rdflib.Literal import Literal
from rdflib.BNode import BNode
from rdflib.Identifier import Identifier
from rdflib.Namespace import Namespace
from rdflib.constants import TYPE, ABOUT
from rdflib.TripleStore import TripleStore

import kraken.Constants
from TypicalNamespaces import *
import kraken.tools.TextUi as TextUi


class EntryBase:
    
    def __init__(self):        
        self.fields = {}
    
    # 
    #  Lists all the properties of the entry  
    #  MIGHT be rewritten by the inheriting classes.
    #
    def listPropertyNames(self):
        return self.fields.keys()
    
    # 
    #  Get a property of the entry.  
    #  MIGHT be rewritten by the inheriting classes.
    #
    def getProperty(self, propertyName):
       return self.fields[propertyName]
    
      
    # 
    #  Adds a new property to the entry. 
    #  MIGHT be rewritten by the inheriting classes.
    #
    def addProperty(self, property_name, property_value):
       self.fields[property_name] = property_value
        
    # 
    #  Returns the resource identifier of the entry 
    #  SHOULD be rewritten by the inheriting classes.
    #    
    def getResourceURI(self):
        return None
    
    # 
    #  Returns the mapping of the fieldname to RDF attribute 
    #  SHOULD be rewritten by the inheriting classes.
    #    
    def getMapping(self, field):
        return field
    
    '''
    Returns a hash of the namespaces (with the abbreviation as a key)
    might be rewritten or extended by the implementing classes.    
    '''
    def getNamespaces(self):
        return namespaces
            
    '''
    Returns the RDF class of the entry
    '''            
    def getRdfClass(self):
        return URIRef(NS_RDF["thing"])            
            
    # 
    #  Create store
    #
    def createStore(self):
       # create the store
       store = TripleStore()
       theClass = self.__class__
       # add the namespaces
       nspaces = self.getNamespaces()
       for ns in nspaces.keys():          
          store.prefix_mapping(ns, nspaces[ns])
       return store 
    
    #
    #    Writing the entry into an RDF file
    #        
    def writeEntryToRdf(self, fileName, store = None):
       if store == None:
          store = self.createStore()
       # add the attributes 
       entry = self.getResourceURI()
       for key in self.listPropertyNames():
          value = self.getProperty(key)
          store.add((entry, self.getMapping(key), Literal(value)))
       # this is a test to make it conform to an owl class
       store.add((entry, URIRef(NS_RDF["type"]), self.getRdfClass()))
       # save and post-process the xml:base stuff which is not supported by RDFLib
       if fileName == None:
           return
       self.saveStoreAndPostProcess(store, fileName)
         
    def saveStoreAndPostProcess(self, store, fileName):
       store.save(fileName + ".tmp")
       FILE = open(fileName + ".tmp")
       TOCOPY = open(fileName,"w")
       change = 1
       while 1:
           line = FILE.readline()
           if line == "":
               break
           if change == 1 and string.find(line, "xmlns:base") != -1:
               line = string.replace(line, "xmlns:base", "xml:base")
               change = 0
           TOCOPY.write(line)
       FILE.close()
       TOCOPY.close()
       os.remove(fileName + ".tmp")
         
    #
    #    Reading the entry from an RDF file which contains only a 
    #    single entry.
    #        
    def readEntryFromRdf(self, fileName):       
       store = self.createStore()
       theClass = self.__class__          
       if (not os.path.exists(fileName)):
          logging.warning(fileName + "does not exist") 
          raise fileName + "does not exist"
       urlName = urllib.pathname2url(fileName)
       store.load(urlName)
       # FIXME
       # Assume that in the store there is only one label, which 
       # is the one representing the entry
       # This is a bit of a difficult assumption and it will need to be fixed
       # later, by THINKING
       # It also prevents it to be adapted to the other entries.
       oneProperty = self.listPropertyNames()[0]
       # for a in store.subjects(predicate=self.getMapping("label")):
       for a in store.subjects(predicate=self.getMapping(oneProperty)):
          subject = a
          #self.label = a
          # FIXME: this might in fact be needed for Entry
          break
       resource = store[URIRef(subject)]
       # and now, fill in the entry from the loaded stuff
       # question: does eval work for all of them?
       #for key in theClass.mappings.keys():
       for key in self.listPropertyNames():
           value = str(resource[self.getMapping(key)])
           typeOfField = type(self.getProperty(key))
           if typeOfField in [TupleType,ListType,DictType,IntType,time.struct_time]:
               try:
                  self.addProperty(key, eval(value))
               except:
                  print sys.exc_info()
                  print "Unexpected value for key " + key + " = " + value
                  print "Can not eval. Keeping the default."                  
           else:
              self.addProperty(key, value)
           # check for errors in types, but accept a tuple instead of struct_time
           newtype = type(self.getProperty(key))
           if  (newtype != typeOfField and 
               not (typeOfField == time.struct_time and newtype == TupleType)):
               print "ERROR: The type of the field changed after load!!!"
               print key, self.getProperty(key)
               
    '''
     Helper function: edit a field different datatypes.     
    '''
    def editField(self, fieldname, prompt = None):
      if prompt == None:
          prompt = fieldname
      val = self.getProperty(fieldname)
      typeOfField = type(val)
      if typeOfField == StringType:
         val2 = TextUi.inputWithDefault(prompt, val)
      elif typeOfField == ListType:
          val2 = TextUi.inputSequence(prompt, val)
      elif typeOfField == IntType:
          val2 = TextUi.inputInteger(prompt, val)
      else:
          print "This datatype" + typeOfField + " can not be currently edited in kraken"
          return
      if val != val2:
          self.addProperty(fieldname, val2)
               
